
# AUTOMATICALLY GENERATED BY 'RELIC':
# * DO NOT EDIT THIS MODULE MANUALLY.
# * DO NOT COMMIT THIS MODULE TO YOUR GIT REPOSITORY

__all__ = [
    '__version__',
    '__version_short__',
    '__version_long__',
    '__version_post__',
    '__version_commit__',
    '__version_date__',
    '__version_dirty__',
    '__build_date__',
    '__build_time__',
    '__build_status__'
]

__version__ = '3.4.12'
__version_short__ = '3.4.12'
__version_long__ = '3.4.12-0-gfd82437d'
__version_post__ = '0'
__version_commit__ = 'fd82437d'
__version_date__ = '2018-02-07 16:36:45 -0500'
__version_dirty__ = False
__build_date__ = '2018-02-14'
__build_time__ = '21:42:02.379382'
__build_status__ = 'release' if not int(__version_post__) > 0 \
    and not __version_dirty__ \
    else 'development'
